(module easy-args (define-arguments)
  (import scheme chicken)
  (use srfi-13)

  (define-syntax define-arguments
    (syntax-rules ()
      ((_ (<name> <value/guard> ...) ...)
       (begin (define <name>) ...
         (let* ((argument-list '()) ; accumulate parameters
                (make-arg (lambda (name #!optional value (guard o))
                            (let ((flag (string-delete #\* (format "-~a" name)))
                                  (parameter (make-parameter value guard))
                                  (type (cond ((boolean? value) 'boolean)
                                              ((string? value) 'string)
                                              ((number? value) 'number)
                                              (else (error 'define-arguments
                                                           "Invalid default value"
                                                           value)))))
                              (set! argument-list
                                (cons (list flag parameter type)
                                      argument-list))
                              parameter))))
           ; define parameter object for each argument
           (set! <name> (make-arg '<name> <value/guard> ...)) ...
           ; process (command-line-arguments) against defined parameters
           (command-line-arguments ; remove any we handle
             (let try-match ((unmatched '()) (lst (command-line-arguments)))
               (if (null? lst)
                 (reverse unmatched)
                 (let ((match (assoc (car lst) argument-list)))
                   (if match
                     (try-match unmatched
                       (case (caddr match)
                         ((boolean) ((cadr match) #t) (cdr lst))
                         ((string) ((cadr match) (cadr lst)) (cddr lst))
                         ((number) ((cadr match)
                                    (string->number (cadr lst)))
                                   (cddr lst))))
                     (try-match (cons (car lst) unmatched)
                                (cdr lst)))))))))))))
